local _, core = ...;
local _G = _G;
local L = core.L;

local DKP = core.DKP;
local GUI = core.GUI;
local Util = core.Util;
local PDKP = core.PDKP;
local Guild = core.Guild;
local Shroud = core.Shroud;
local Defaults = core.defaults;
local Comms = core.Comms;
local Setup = core.Setup;
local Raid = core.Raid;

local AceGUI = LibStub("AceGUI-3.0")

Shroud.window = nil;

Shroud.windowPosition = nil

Shroud.shrouders = {
    names={},
    table={},
    database_name='',
};

Shroud.shroudPhrases = {
    ['shrouding']=true,
    ['shroud']=true,
    ['thirst']=true,
    ['thirsting']=true,
}

function Shroud:UpdateWindow()
    if Shroud.window == nil then
        Util:Debug('Setting up shrouding window')
        Setup:ShroudingWindow()
    end

    local scroll = Shroud.window.scroll

    scroll:ReleaseChildren() -- Clear the previous entries.

    Util:Debug('Updating the shrouding window')

    local shrouders = Shroud.shrouders.table

    local dbName = Shroud.shrouders.database_name;

    if dbName ~= nil and dbName ~= '' then
       _G['pdkp_shrouding_window_title']:SetText(dbName .. ' Shrouding')
    end

    local function compareDesc(a,b) return a.dkpTotal < b.dkpTotal end
    local function compareAsc(a,b) return a.dkpTotal > b.dkpTotal end
    table.sort(shrouders, compareDesc)

    local br = "\n"
    local labels = { AceGUI:Create("Label"), AceGUI:Create("Label"), AceGUI:Create("Label") }
    local colors = {'00FF96', 'FFF569', 'FF3F40'} -- Winner, Tied, Loser colors
    local text = {'[WINNER]', '[TIED]', br .. '[SHROUDERS]' }

    for i=1, #labels do
        local text = Util:FormatFontTextColor(colors[i], text[i] .. br)
        labels[i]:SetText(text)
    end

    local winnerLabel, tiedLabel, loserLabel = labels[1], labels[2], labels[3]

    local winners, losers = {}, {}

    local maxTotalDKP = shrouders[#shrouders].dkpTotal

    for i=1, #shrouders do
        local member = shrouders[i]
        if member.dkpTotal > 0 then -- Ignore people with 0 DKP. They can't shroud on anything anyway.
            local sg, nameLabel, dkpLabel = AceGUI:Create("SimpleGroup"), AceGUI:Create("Label"), AceGUI:Create("Label")
            sg:SetLayout("Flow")
            sg:SetFullWidth(true)
            nameLabel:SetFullWidth(false)
            dkpLabel:SetFullWidth(false)

            nameLabel:SetWidth(98)
            dkpLabel:SetWidth(30)

            local coloredDKP = Util:FormatFontTextColor('FFDF00', member.dkpTotal)

            nameLabel:SetText(member.name)
            dkpLabel:SetText(coloredDKP)

            sg:AddChild(nameLabel)
            sg:AddChild(dkpLabel)

            if #shrouders == 1 then -- We only have one shrouder. They are the winner by default.
                table.insert(winners, sg)
            else -- we have more than one shrouder...
                local dkpTotal = member.dkpTotal
                if dkpTotal == maxTotalDKP then
                    table.insert(winners, sg)
                elseif dkpTotal < maxTotalDKP then
                    table.insert(losers, sg)
                end
            end
            sg.dkpTotal = member.dkpTotal -- So we can resort the table later on.
        end
    end

    local function AddChildren(label, table)
        scroll:AddChild(label)
        for i=1, #table do scroll:AddChild(table[i]) end
    end

    -- Add the labels.
    if #winners == 1 then AddChildren(winnerLabel, winners) end
    if #winners > 1 then AddChildren(tiedLabel, winners) end
    if #losers > 0 then
        table.sort(losers, compareAsc)
        AddChildren(loserLabel, losers)
    end

    Shroud:SetWindowPosition()

    Shroud.window:Show()
end

-- Updates the shrouding table to reflect the ML's DKP totals.
function Shroud:UpdateShrouders(playerName) -- Only the ML should be able to access this, ideally.
--    if core.Defaults:IsDebug() then Shroud.shrouders = testShrouders end
    local shrouders = Shroud.shrouders

    local player = { name=playerName, dkpTotal=DKP:GetPlayerDKP(playerName) }
    if shrouders.names[playerName] == nil then -- player isn't in the table yet.
        if player.dkpTotal > 0 then
            Util:Debug('Adding shrouder!');
            table.insert(shrouders.names, playerName)
            table.insert(shrouders.table, player);
        end
    else
        for i=1, #shrouders.table do
            local shrouder = shrouders.table[i];
            if shrouder.name == playerName then
                shrouder.dkpTotal = DKP:GetPlayerDKP(playerName)
            end
        end
    end

    if player.dkpTotal > 0 then
        shrouders.names[playerName] = true;
        Shroud.shrouders.database_name = DKP.dkpDB.currentDB;

        if _G['pdkp_shrouding_window_title']:GetText() ~= DKP.dkpDB.currentDB .. ' Shrouding' then
            Shroud:ClearShrouders()
            _G['pdkp_shrouding_window_title']:SetText(DKP.dkpDB.currentDB .. ' Shrouding')
            return Shroud:UpdateShrouders(playerName)
        end

        core.Comms:SendShroudTable()
    end
end

function ShroudWindowPosChanged()
    Shroud.windowPosition = {
        bottom=math.floor(Shroud.window:GetBottom() + 0.5),
        left=math.floor(Shroud.window:GetLeft() + 0.5),
    }
end

function Shroud:SetWindowPosition()
    -- Might fix the shrouding window being huge?
    if Shroud.windowPosition == nil then
        Shroud.window:SetHeight(200);
        Shroud.window:SetHeight(200);
        Shroud.window:SetPoint('BOTTOMLEFT', 0, 0);
    else
        print('Found positioning set')
        Shroud.window:SetPoint("BOTTOMLEFT", Shroud.windowPosition.left, Shroud.windowPosition.bottom)
    end
end

-- Resets the shrouding table.
function Shroud:ClearShrouders()
    Shroud.shrouders = {
        names={},
        table={},
        database_name='',
    };

    if Shroud.window then
        Shroud.window:Hide()
        local scroll = Shroud.window.scroll
        scroll:ReleaseChildren() -- Clear the previous entries.
    end

    -- The ML should sent out a comm command that wipes everyones tables after this is closed.
    if Raid:isMasterLooter() then
        Comms:ClearShrouders()
    end
end

function Shroud:Hide()
    Shroud.window:Hide();
end

function Shroud:Announce()
    PDKP:Print("Congrats", UnitName('PLAYER'));
end

function shrouding_template_function_calls(funcName)
    if funcName == 'pdkp_shrouding_hide' then return Shroud:Hide() end;
    if funcName == 'pdkp_announce_winner' then return Shroud:Announce() end;
end
